/********************************************************************
*               MINI-COMPILADOR PARA O MINI-PASCAL                  *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                              *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*          COMPILADORES: Prof. Mrio Luis R. Oliveira               *
* ----------------------------------------------------------------- *
* Desenvolvida inicialmente para a disciplina de Linguagens Formais *
*         e autmatos. Prof. Jones Oliveira Albuquerque.            *
*                                                                   *
*  Anderson de Rezende Rocha, Flvio Luiz Alves, Jlio Csar Alves  *
*                       Rodrigo Nazar Leite                        *
********************************************************************/

// Esta classe define um estado de um automato

#ifndef ESTADO_H
#define ESTADO_H

#include "lista.h"
#include "ligacao.h"
#include <string>

using namespace std;

// Diretiva para o compilador saber que futuramente encontrara esta classe
template <class TSimbolo>
class clLigacao;

// A classe estado  template em relacao ao simbolo
template <class TSimbolo>
class clEstado
{
    public:
        // Construtor padrao
        clEstado<TSimbolo>();
        // Construtor a partir de um identificador numerico, um identificador indicando se o
        // estado eh final ou nao e um vetor de chars que ser a mensagem de saida, ja que
        // a abordagem deste projeto utiliza maquina de moore.
        clEstado<TSimbolo>(int idEstado, bool ehFinal, char saidaEstado[36]);
        // Construtor de copia
        clEstado<TSimbolo>(const clEstado<TSimbolo>& estado);
        // Destrutor padrao, para desalocar os objetos que usam alocacao dinamica
        ~clEstado<TSimbolo>();
        // Retorna o identificador deste estado.
        int getId();
        // Retorna a mensagem (saida) deste estado
        char *getSaida();
        // Retorna se este estado  final ou nao final.
        bool getEhFinal();
        // Insere uma producao deste estado, com o simbolo passado por parametro,
        // para o novo estado passado tambm por parametro
        bool InserirProducao(TSimbolo *simbolo, clEstado<TSimbolo>*estado);
        // Insere uma producao deste estado, com o simbolo passado por parametro,
        // para o novo estado passado tambm por parametro e uma saida da Producao
        bool InserirProducao(TSimbolo *simbolo, clEstado<TSimbolo>*estado, string saida);
        // Consome um simbolo em especifico, retornando o novo estado
        // NULL se nao existe producao
        clEstado<TSimbolo> *Consumir(TSimbolo *simbolo);
        // Consome um simbolo em especifico, retornando o novo estado
        // NULL se nao existe producao, e a saida da Ligacao por dentro
        clEstado<TSimbolo> *Consumir(TSimbolo *simbolo, string &saidaDaLigacao);
        // Ajusta qual o proximo estado que este aponta.
        void setProximoEstado(clEstado<TSimbolo> *estado);
        // Retorna o proximo estado.
        clEstado<TSimbolo> *getProximoEstado();

        // devolve a lista de producoes
        const clLista<clLigacao<TSimbolo> > *getProducoes();

    private:
        // Identificador do estado
        int id;
        // Saida do estado
        char saida[36];
        // Lista com todas as producoes deste estado. Melhor dizendo e uma lista
        // de ligacoes para outros estados mediante simbolos especificos.
        clLista<clLigacao<TSimbolo> > *producoes;
        // Guarda se este um estado final ou nao final.
        bool ehFinal;
        // Ponteiro que armazena internamente qual o proximo estado
        // na estrutura do automato que esta sendo produzido.
        clEstado<TSimbolo> *proximoEstado;
};

// Construtor padrao
template <class TSimbolo>
clEstado<TSimbolo>::clEstado<TSimbolo>()
{
    proximoEstado = NULL;
    producoes = new clLista<clLigacao<TSimbolo> >();
}

// Construtor a partir do identificador, booleano final/nao final e o vetor
// de chars que sera a mensagem que este estado produzira.
template <class TSimbolo>
clEstado<TSimbolo>::clEstado<TSimbolo>(int idEstado, bool ehFinal,char saidaEstado[36])
{
    id = idEstado;
    clEstado<TSimbolo>::ehFinal = ehFinal;
    strcpy(saida, saidaEstado);
    proximoEstado = NULL;
    producoes = new clLista<clLigacao<TSimbolo> >();
}

// Construtor de copia
template <class TSimbolo>
clEstado<TSimbolo>::clEstado<TSimbolo>(const clEstado<TSimbolo>& estado)
{
    id = estado.id;
    strcpy(saida, estado.saida);
    producoes = estado.producoes;
    ehFinal = estado.ehFinal;
    proximoEstado = estado.proximoEstado;
}

// Destroi tudo que foi alocado dinamicamente
template <class TSimbolo>
clEstado<TSimbolo>::~clEstado<TSimbolo>()
{
    delete producoes;
}

// Retorna o identificador deste estado
template <class TSimbolo>
int clEstado<TSimbolo>::getId()
{
    return id;
}

// Retorna a mensagem armazenada por este estado
template <class TSimbolo>
char *clEstado<TSimbolo>::getSaida()
{
    return saida;
}

// Retorna se este e um estado final ou nao final
template <class TSimbolo>
bool clEstado<TSimbolo>::getEhFinal()
{
    return ehFinal;
}

// Insere uma producao deste estado para outro a partir de um simbolo especifico
template <class TSimbolo>
bool clEstado<TSimbolo>::InserirProducao(TSimbolo *simbolo, clEstado<TSimbolo> *estado)
{
    return InserirProducao(simbolo, estado, "");
}

// Insere uma producao deste estado para outro a partir de um simbolo especifico e uma saida
template <class TSimbolo>
bool clEstado<TSimbolo>::InserirProducao(TSimbolo *simbolo, clEstado<TSimbolo> *estado, string saida)
{
    clLigacao<TSimbolo> *ligacao;
    ligacao = new clLigacao<TSimbolo>(*simbolo, estado, saida);
    producoes->InserirNoFim(*ligacao);
    return true;
}

// Consome um simbolo do alfabeto passado por parametro
template <class TSimbolo>
clEstado<TSimbolo> *clEstado<TSimbolo>::Consumir(TSimbolo *simbolo)
{
    string temp;
    return Consumir(simbolo, temp);
}

// Consome um simbolo do alfabeto passado por parametro e retorna por dentro a saida da ligacao
template <class TSimbolo>
clEstado<TSimbolo> *clEstado<TSimbolo>::Consumir(TSimbolo *simbolo, string &saidaDaLigacao)
{
    // Uma ligacao deve ser considerada igual a outra se apenas o simbolo for igual
    clLigacao<TSimbolo> *ligacao;
    ligacao = new clLigacao<TSimbolo>(*simbolo);
    // o metodo Encontrar da classe lista, encontra um valor e o atualiza na varivel passada.
    // No nosso caso, por exemplo, foi criada uma ligacao com o simbolo recebido e destino == NULL.
    // Como uma ligacao eh igual a outra somente em relacao ao simbolo, o metodo Encontrar
    // ao encontrar uma ligacao com este simbolo a atualiza com o destino correto
    if (!producoes->Encontrar(*ligacao))
    {
        delete ligacao;
        return (NULL);
    }
    else
    {
        clEstado<TSimbolo> *destino(ligacao->getDestino());
        saidaDaLigacao = ligacao->getSaida();
        delete ligacao;
        return  (destino);
    }
}

// Escolhe quem  o proximo estado na estruturao fisica do automato
template <class TSimbolo>
void clEstado<TSimbolo>::setProximoEstado(clEstado<TSimbolo> *estado)
{
    proximoEstado = estado;
}

// Retorna quem  o proximo estado na estrutura interna do automato
template <class TSimbolo>
clEstado<TSimbolo> *clEstado<TSimbolo>::getProximoEstado()
{
    return proximoEstado;
}

// retorna a lista de producoes do estado
template <class TSimbolo>
const clLista<clLigacao<TSimbolo> > *clEstado<TSimbolo>::getProducoes()
{
    return producoes;
}
#endif

