/********************************************************************
*               MINI-COMPILADOR PARA O MINI-PASCAL                  *
* ----------------------------------------------------------------- *
*      Membros: Anderson de Rezende Rocha - DCC/UFLA                *
*               undersun@comp.ufla.br                              *
*                                                                   *
*               Jlio Csar Alves - DCC/UFLA                        *
*               jcalves@comp.ufla.br                                *
* ----------------------------------------------------------------- *
*          COMPILADORES: Prof. Mrio Luis R. Oliveira               *
********************************************************************/


/* OBSERVAES:
    + Cada token aponta para o proximo de acordo com a sequencia encontrada no arquivo (nao esta sendo utilizado)
    + GetValue() pode parecer prolixo mas isto no procede, dado que, esta foi uma
      forma de padronizar algumas funes para o analisador de ERROS.


   MTODOS:
    + clToken();
    + clToken(string umAtomo, FLAGS umTipo, string umAtributo, int umaLinha, clToken *umProximo = NULL);
    + clToken(const clToken &token);
    + ~clToken();
    + string GetAtomo();
    + FLAGS GetTipo();
    + string GetAtributo();
    + int GetLinha();
    + clToken *GetProximo();
    + string GetValue();
    + void SetTodos(string umAtomo, FLAGS umTipo, string umAtributo, int umaLinha, clToken *umProximo = NULL);
    + void SetAtomo(string umAtomo);
    + void SetTipo(FLAGS umTipo);
    + void SetAtributo(string umAtributo);
    + void SetLinha(int umaLinha);
    + void SetProximo(clToken *token);
    + void Imprimir();
*/

using namespace std;

#ifndef TOKEN_H
#define TOKEN_H

#include<iostream>
#include<string>

class clToken
{
    public:
        // Construtor padro
        clToken();
        // Constri um Token com todos os seus atributos j selecionados
        clToken(string umAtomo, FLAGS umTipo, string umAtributo, int umaLinha, clToken *umProximo = NULL);
        // Construtor de cpia
        clToken(const clToken &token);
        // Destrutor
        ~clToken();
        // Retorna o atomo do token
        string GetAtomo();
        // Retorna o FLAG identificador do token
        FLAGS GetTipo();
        // Retorna o atributo armazenado pelo token
        string GetAtributo();
        // Retorna a linha em que tal token ocorre
        int GetLinha();
        // Referncia para o prximo token. Importante para se
        // construir uma lista de tokens
        clToken *GetProximo();
        // Retorna uma cpia para o valor do tomo
        string GetValue();
        // Seta todos os atributos do token atual
        void SetTodos(string umAtomo, FLAGS umTipo, string umAtributo, int umaLinha, clToken *umProximo = NULL);
        // Seta apenas o atributo 'atomo' para o token
        void SetAtomo(string umAtomo);
        // Seta o 'tipo' do token atual
        void SetTipo(FLAGS umTipo);
        // Seta o atributo do token atual
        void SetAtributo(string umAtributo);
        // Seta a linha em que aparece o token
        void SetLinha(int umaLinha);
        // Seta o prximo token apontado pelo atual
        void SetProximo(clToken *token);
        // Imprime na sada padro o token atual
        void Imprimir();
    private:
        string atomo;
        FLAGS tipo;
        string atributo;
        int linha;
        clToken *proximo;
};

// Construtor padro
clToken::clToken()
{
    tipo = INVALIDO;
    linha = 0;
}

// Constri um Token com todos os seus atributos j selecionados
clToken::clToken(string umAtomo, FLAGS umTipo, string umAtributo, int umaLinha, clToken *umProximo)
{
    SetTodos(umAtomo, umTipo, umAtributo, umaLinha, umProximo);
}

// Construtor de cpia
clToken::clToken(const clToken &token)
{
    SetTodos(token.atomo, token.tipo, token.atributo, token.linha, token.proximo);
}

// Seta todos os atributos do token atual
void clToken::SetTodos(string umAtomo, FLAGS umTipo, string umAtributo, int umaLinha, clToken *umProximo)
{
    SetAtomo(umAtomo);
    SetTipo(umTipo);
    SetLinha(umaLinha);
    SetAtributo(umAtributo);
    SetProximo(umProximo);
}

// Destrutor
clToken::~clToken(){}

string clToken::GetAtomo()
{
    return atomo;
}

// Retorna o FLAG identificador do token
FLAGS clToken::GetTipo()
{
    return tipo;
}

// Retorna o atributo armazenado pelo token
string clToken::GetAtributo()
{
    return atributo;
}

// Retorna a linha em que tal token ocorre
int clToken::GetLinha()
{
    return linha;
}

// Referncia para o prximo token. Importante para se 
// construir uma lista de tokens
clToken *clToken::GetProximo()
{
    return proximo;
}

// Retorna uma cpia para o valor do tomo para manter compatibilidade
// com o autmato
string clToken::GetValue()
{
    return GetAtomo();
}

// Seta apenas o atributo 'atomo' para o token
void clToken::SetAtomo(string umAtomo)
{
    atomo = umAtomo;
}

// Seta o 'tipo' do token atual
void clToken::SetTipo(FLAGS umTipo)
{
    tipo = umTipo;
}

// Seta o atributo do token atual
void clToken::SetAtributo(string umAtributo)
{
    atributo = umAtributo;
}

// Seta a linha em que aparece o token
void clToken::SetLinha(int umaLinha)
{
    linha = umaLinha;
}

// Seta o prximo token apontado pelo atual
void clToken::SetProximo(clToken *token)
{
    proximo = token;
}

// Imprime na sada padro o token atual
void clToken::Imprimir()
{
    cout << "atomo: " << GetAtomo() << " tipo: " << GetTipo() << " atributo: "
         << GetAtributo() << " linha: " << GetLinha() << endl;
}

#endif

